import argparse

import browser_cookie3

import pyscrapers.core.utils
from pyscrapers.photos.facebook import scrape_facebook
from pyscrapers.photos.instagram import scrape_instagram
from pyscrapers.photos.mambaru import scrape_mambaru
from pyscrapers.photos.travelgirls import scrape_travelgirls
from pyscrapers.photos.vk import scrape_vk

import pylogconf.core


def main():
    pylogconf.core.setup()
    # command line parsing
    parser = argparse.ArgumentParser(
        description='''download photos from various sites'''
    )
    choices = ["facebook", "instagram", "travelgirls", "vk", "mamba.ru"]
    parser.add_argument(
        '-t',
        '--type',
        help="Which site to download from",
        choices=choices,
    )
    parser.add_argument(
        '-u',
        '--user_id',
        help='''id of the user to download the albums of
            https://www.facebook.com/profile.php?id=[user_id]
            https://www.instagram.com/[user_id]
            http://www.travelgirls.com/member/[user_id]
            https://vk.com/id[user_id]
            http://www.mamba.ru/mb[user_id]
        ''',
    )
    parser.add_argument(
        '-d',
        '--debug',
        help='debug requests',
        default=False,
        action='store_true',
    )
    parser.add_argument(
        '-s',
        '--start',
        help='start number for image names',
        type=int,
        default=0,
    )
    args = parser.parse_args()
    if args.user_id is None:
        parser.error('-u/--user_id must be given')
    if args.debug:
        pyscrapers.core.utils.debug_requests()

    # load cookies from browser
    cookies = browser_cookie3.firefox()

    urls = []
    if args.type == "facebook":
        urls = scrape_facebook(args.user_id, cookies)
    if args.type == "instagram":
        urls = scrape_instagram(args.user_id, cookies)
    if args.type == "travelgirls":
        urls = scrape_travelgirls(args.user_id, cookies)
    if args.type == "vk":
        urls = scrape_vk(args.user_id, cookies)
    if args.type == "mamba.ru":
        urls = scrape_mambaru(args.user_id, cookies)
    pyscrapers.core.utils.download_urls(urls, start=args.start)


if __name__ == '__main__':
    main()



import logging
from typing import List

import requests
from lxml import etree

import pyscrapers.core.utils


def scrape_facebook(user_id: str, cookies) -> List[str]:
    logger = logging.getLogger(__name__)
    s = requests.Session()
    s.cookies = cookies

    url = 'https://www.facebook.com/{user_id}/photos'.format(user_id=user_id)
    logger.debug('url is [%s]', url)
    r = s.get(url)
    root = pyscrapers.core.utils.get_html_dom_content(r)
    # print(etree.tostring(root, pretty_print=True))
    # sys.exit(1)

    urls = []
    e_a = root.xpath('//img')
    for x in e_a:
        print(etree.tostring(x, pretty_print=True))
    return urls



"""
How does this work?
When you fetch the page of a user on instagram you get an html with a javascript embedded
in it with a json object embedded in that. This json object describes the user, his id,
his profile photo and the first 12 images for that user.
If you want to more photos you have to do a follow-up AJAX request to the server.
"""
import json
import logging
from typing import List

import requests
from lxml import etree

import pyscrapers.core.utils


def scrape_instagram(user_id: str, cookies) -> List[str]:
    logger = logging.getLogger(__name__)
    # constants
    domain = 'www.instagram.com'
    base = 'https://{domain}'.format(domain=domain)

    url = '{base}/{user_id}/'.format(base=base, user_id=user_id)
    logger.debug('url is [%s]', url)

    # start a session
    s = requests.Session()
    s.cookies = cookies

    r = s.get(url)
    root = pyscrapers.core.utils.get_html_dom_content(r)
    # scrape.utils.print_element(root)

    urls = []
    # register regular expressions with lxml
    # this means that we can use regular expression functions like 'match'
    # by specifying 're:match' in our xpath expressions
    ns = etree.FunctionNamespace("http://exslt.org/regular-expressions")
    ns.prefix = 're'
    e_a = root.xpath('//script[re:match(text(), "^window._sharedData")]')
    assert len(e_a) == 1
    e_a = e_a[0]
    data = e_a.text
    json_text = data[data.find('{'):data.rfind('}') + 1]
    d = json.loads(json_text)
    my_list = d['entry_data']['ProfilePage']
    assert (len(my_list) == 1)
    c = my_list[0]["user"]
    if 'profile_pic_url_hd' in c:
        urls.append(c['profile_pic_url_hd'])
    elif 'profile_pic_url' in c:
        urls.append(c['profile_pic_url'])
    user_id = c['id']
    # json.dump(c, sys.stdout, indent=4)
    # list_node = c['media']['nodes']
    # for x in list_node:
    #    urls.append(x['display_src'])

    # now we need to do the follow up query
    # url2 = '{base}/query/'.format(base=base)

    """ g_user(278094193)+{+media.after(732083027810814056,+12)+{++count,++nodes+{++++caption,++++code,
    ++++comments+{++++++count++++},++++comments_disabled,++++date,++++dimensions+{++++++height,++++++width++++},
    ++++display_src,++++id,++++is_video,++++likes+{++++++count++++},++++owner+{++++++id++++},++++thumbnail_src,
    ++++video_views++},++page_info}+}" """
    """
    data = {
        # the 5000 is the number of images you want (big number to get all)
        # the 0 in media.after is after what. 0 seems to return everything I think
        # 'q':'ig_user({0})'.format(user_id)+'{ media.after(0, 5000) { count, nodes { display_src } } }',
        'q': 'ig_user({0})'.format(user_id)+'{ media.after(0, 5000) { nodes { display_src } } }',
        # 'q':'ig_user({0})'.format(user_id)+' { media.after(0, 5000) { count, nodes { caption, code, comments
        # { count }, comments_disabled, date, dimensions { height, width }, display_src, id, is_video, likes { count },
        # owner { id }, thumbnail_src, video_views }, page_info } }',
        'ref': 'users::show',
        # this is constant as far as I can tell
        'query_id': '17842962958175392',
    }
    cookie_to_search = 'csrftoken'
    cookie_value = r.cookies[cookie_to_search]
    headers = {
        # these two are necessary or you wont get response from instagram
        'X-CSRFToken': r.cookies[cookie_to_search],
        'Referer': url,
    }
    # you must send cookies and headers to get the data...
    r2 = s.post(url2, data=data, headers=headers)
    for node in res['media']['nodes']:
        urls.append(node['display_src'])
    """

    url2 = '{base}/graphql/query/'.format(base=base)
    params = {
        'query_id': 17888483320059182,
        'variables': json.dumps({
            'id': user_id,
            'first': 5000,
        })
    }
    r2 = s.get(url2, params=params, cookies=r.cookies)
    root = pyscrapers.core.utils.get_html_dom_content(r2)
    res = json.loads(root.text)
    for node in res['data']['user']['edge_owner_to_timeline_media']['edges']:
        urls.append(node['node']['display_url'])
    return urls



"""
This module is s set of utilities for this entire project
"""


import http.client
import logging
import os
import shutil
import urllib.parse

import requests
from lxml import etree, html

from pyscrapers.core import ffprobe


def print_cookies(cookies, domain):
    """
    Print the cookies from a specific domain
    :param cookies:
    :param domain:
    :return:
    """
    print(domain)
    for cookie in cookies:
        print(cookie)


def get_http_status_string(code: int):
    """
    This function returns a description of an HTTP status code (404 - not found etc).
    Unfortunately, the requests module does not provide a clean API for this so
    we must access a protected member (underscore member) of 'requests.status_code'.
    See:
    https://stackoverflow.com/questions/24718557/get-the-description-of-a-status-code-in-python-requests
    :param code:
    :return:
    """
    # noinspection PyProtectedMember
    return "http code [{}], [{}]".format(code, requests.status_codes._codes[code][0])


def get_html_dom_content(response):
    """
    Get the content from a request
    :param response:
    :return:
    """
    assert response.status_code == 200, get_http_status_string(response.status_code)
    str_content = response.content.decode()
    root = html.fromstring(str_content)
    return root


def download_urls(urls, start=0):
    """
    Download a list of urls
    :param urls:
    :param start:
    :return:
    """
    logger = logging.getLogger(__name__)
    counter = start
    logger.info('got [%d] real urls', len(urls))
    for url in urls:
        logger.info('downloading [%s]...', url)
        response = requests.get(url, stream=True)
        assert response.status_code == 200, response.content
        filename = 'image{0:04}.jpg'.format(counter)
        assert not os.path.isfile(filename)
        with open(filename, 'wb') as file_handle:
            response.raw.decode_content = True
            shutil.copyfileobj(response.raw, file_handle)
        logger.info('written [%s]...', filename)
        counter += 1


def debug_requests():
    """
    Activate the debugging features of the requests module
    :return:
    """
    http.client.HTTPConnection.debuglevel = 1
    requests_log = logging.getLogger("requests.packages.urllib3")
    requests_log.propagate = True


def add_http(url, main_url):
    """
    add two urls together
    :param url:
    :param main_url:
    :return:
    """
    return urllib.parse.urljoin(main_url, url)


def print_element(element):
    """
    from xml elements from etree
    :param element:
    :return:
    """
    print(etree.tostring(element, pretty_print=True).decode())


def download_url(source: str, target: str) -> None:
    """
    Download a single url to a file
    """
    logger = logging.getLogger(__name__)
    logger.info('downloading [%s] to [%s]', source, target)
    if os.path.isfile(target):
        logger.info('skipping [%s]', target)
        return
    try:
        response = requests.get(source, stream=True)
        assert response.status_code == 200, response.content
        with open(target, 'wb') as file_handle:
            response.raw.decode_content = True
            shutil.copyfileobj(response.raw, file_handle)
    except IOError:
        os.unlink(target)
    logger.info('written [%s]...', target)


FAIL = True


def download_video_if_wider(source: str, target: str, width: int) -> bool:
    """
    Download a video if it is wider than a certain width
    :param source:
    :param target:
    :param width:
    :return:
    """
    logger = logging.getLogger(__name__)
    logger.info('downloading [%s] to [%s]', source, target)
    if os.path.isfile(target):
        file_width = ffprobe.height(target)
        if file_width >= width:
            logger.info('skipping because video with width exists [%s] %s %s', target, file_width, width)
            return True
        else:
            logger.info('continuing with download because of width [%s] %s %s', target, file_width, width)
    try:
        response = requests.get(source, stream=True)
        if FAIL:
            assert response.status_code == 200, response.content
        else:
            if response.status_code != 200:
                logger.info("got bad error code [%s] and failed to download", response.status_code)
                return False
        with open(target, 'wb') as file_handle:
            response.raw.decode_content = True
            shutil.copyfileobj(response.raw, file_handle)
    except IOError:
        if os.path.isfile(target):
            os.unlink(target)
        if FAIL:
            raise ValueError("count not download")
        else:
            logger.info("failed to download file")
            return False
    logger.info('written [%s]...', target)
    return True


class Urls:
    """ list of urls to download and to where """
    def __init__(self, download_as_collecting=False):
        """
        constructor
        :param download_as_collecting:
        """
        self.list = []
        self.download_as_collecting = download_as_collecting

    def add_url(self, source, target):
        """
        add url to the list
        :param source:
        :param target:
        :return:
        """
        self.list.append((source, target))
        if self.download_as_collecting:
            download_url(source=source, target=target)

    def print(self):
        """
        print the list
        :return:
        """
        for (source, target) in self.list:
            print(source, target, sep="\t")

    def download(self):
        """
        download the list
        :return:
        """
        for (source, target) in self.list:
            download_url(source=source, target=target)



import setuptools

"""
The documentation can be found at:
http://setuptools.readthedocs.io/en/latest/setuptools.html
"""
setuptools.setup(
    # the first three fields are a must according to the documentation
    name='pyscrapers',
    version='0.0.8',
    packages=[
        'pyscrapers',
        'pyscrapers.core',
        'pyscrapers.scripts',
        'pyscrapers.photos',
    ],
    # from here all is optional
    description='A collection of scrapers for the web',
    long_description='project to produce various useful scrapers',
    author='Mark Veltzer',
    author_email='mark.veltzer@gmail.com',
    maintainer='Mark Veltzer',
    maintainer_email='mark.veltzer@gmail.com',
    keywords=[
        'scrape',
        'images',
        'social',
        'facebook',
        'instagram',
        'vk.com',
        'download',
        'pics',
    ],
    url='https://veltzer.github.io/pyscrapers',
    download_url='https://github.com/veltzer/pyscrapers',
    license='MIT',
    platforms=[
        'python3',
    ],
    install_requires=[
        'lxml',
        'requests',
        'browser-cookie3',
        'pylogconf',
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3 :: Only',
        'Topic :: Utilities',
    ],
    data_files=[
    ],
    entry_points={'console_scripts': [
        'pyscrapers_photos=pyscrapers.scripts.photos:main',
    ]},
    python_requires='>=3',
)
